class JiraIssue
  module TriageTracker
    def set_sla_triaged_at!(sla_triaged = nil)
      sla_triaged ||= triaged
      return if sla_triaged.nil?
      jira_issue = jiralicious_issue
      jira_issue.fields.set(SLA_TRIAGED_AT_FIELD, sla_triaged.iso8601)
      jira_issue.save!
    end

    def set_over_triage_sla!(over_triage = nil)
      over_triage ||= over_triage_sla?
      return if over_triage.nil? || over_triage == false
      jira_issue = jiralicious_issue
      jira_issue.fields.set_id(OVER_TRIAGE_SLA_FIELD, OVER_TRIAGE_SLA_FIELD_YES)
      jira_issue.save!
    end

    def time_to_triage
      if triaged
        triaged - sla_base
      else
        nil
      end
    end

    def triage_sla_target
      case priority
      when :p1
        sla_base + 0.5.hours
      when :p2
        sla_base + 1.hours
      when :p3
          2.business_hours.after(sla_base)
      when :p4
          1.business_day.after(sla_base)
      else
        nil
      end
    end

    def triage_sla_total_available_time
      triage_sla_target - sla_base
    end

    def triage_sla_remaining_time
      triage_sla_target - Time.now
    end

    def over_triage_sla?
      case
      when over_triage_sla == API_OVER_TRIAGE_SLA_FIELD_NO
        false
      when triage_sla_target.nil?
        false
      when triaged.nil? && Time.now > triage_sla_target
        true
      when triaged.nil? && Time.now < triage_sla_target
        false
      when sla_triaged_at.nil? && triaged < triage_sla_target
        false
      when sla_triaged_at.nil? && triaged > triage_sla_target
        true
      when triaged && sla_triaged_at > triage_sla_target
        true
      end
    end
  end
end
